/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

window.ScoTools = (function() {
var ScoTools = {};

/**
* Klasse, die grundlegende Funktionen zur Kommunikation mit einer SCORM-API bereitstellt (nach der Initialisierung muss start() aufgerufen werden!)
*/
var LMSBasicLink = function() {
    Object.defineProperty(this, 'api', { value: LMSBasicLink.findAPI(), writeable: false, enumerable: true });
}

// Aktueller Status: -1 => Noch nicht initialisiert, 0 => Bereit, 1 => Beendet
LMSBasicLink.prototype.status = -1;

// Kann die Schnittstelle (noch) verwendet werden?
Object.defineProperty(LMSBasicLink.prototype, 'ready', { get() { return !this.status; }, enumerable: true } );

/**
* API auffinden
* @return Object    LMS-API
*/
LMSBasicLink.findAPI = function() {
    for (var w = window; w; w = (w.parent == w) ? null : w.parent) {
        if (w.API && (typeof w.API.LMSInitialize == "function"))
            return w.API;
    }
    return null;
}

/**
* LMS-Rückgabewert v in Boolean-Wert umwandeln
* @return Boolean   Umgewandelter Wert
*/
LMSBasicLink.lms2bool = function(
    v       // Umzuwandelnder Rückgabewert von API
) {
    if (v == "true")
        return true;
    if (!v)
        return undefined;
    return false;
}

/**
* Was soll nach dem Starten sofort gemacht werden?
* @return undefined
*/
LMSBasicLink.prototype._afterStart = function() {}

/**
* Was soll noch direkt vor dem Beenden gemacht werden?
* @return undefined
*/
LMSBasicLink.prototype._beforeStop = function() {}

/**
* Initialisierung durchführen, sofern dies noch aussteht
* @return Boolean       Wurde (jetzt oder zuvor) eine erfolgreiche Initialisierung durchgeführt
*/
LMSBasicLink.prototype.start = function() {
    if (this.status >= 0)
        return true;
    if (!this.api)
        return false;
    if (LMSBasicLink.lms2bool(this.api.LMSInitialize(""))) {
        this.status = 0;
        this._afterStart();
        return true;
    }
    return false;
}

/**
* Kommunikation mit LMS beenden, sofern dies noch aussteht
* @return Boolean       Wurde (jetzt oder zuvor) die Kommunikation förmlich beendet?
*/
LMSBasicLink.prototype.stop = function() {
    if (this.status < 0)
        return false;
    if (this.status > 0)
        return true;
    if (!this.api)
        return false;
    if (LMSBasicLink.lms2bool(this.api.LMSFinish(""))) {
        this.status = 1;
        this._beforeStop();
        return true;
    }
    return false;
}

// Nummer des letzten Fehlers (als Number)
Object.defineProperty(LMSBasicLink.prototype, 'lastError', { enumerable: true,
    get() {
        if (!this.api)
            return -1;
        return +this.api.LMSGetLastError();
    }
});

// Beschreibung des letzten Fehlers
Object.defineProperty(LMSBasicLink.prototype, 'lastErrorDesc', { enumerable: true,
    get() {
        if (!this.api)
            return 'Die Schnittstelle zur Kommunikation mit dem übergeordneten System konnte nicht gefunden werden.';
        return this.api.LMSGetErrorString(this.api.LMSGetLastError());
    }
});

// Weitergehende Erklärum zum letzten Fehlers
Object.defineProperty(LMSBasicLink.prototype, 'lastErrorHelp', { enumerable: true,
    get() {
        if (!this.api)
            return '';
        return this.api.LMSGetDiagnostic(this.api.LMSGetLastError());
    }
});

/**
* Den Wert von key auslesen
* @return String    Gelesener Wert
*/
LMSBasicLink.prototype.get = function(
    key         // Name des auszulesenden Wertes
) {
    if (this.status)
        return undefined;
    var result = this.api.LMSGetValue(key);
    if ((result == "") && this.lastError)
        return undefined;
    return result;
}

/**
* Dem LMS mitteilen, dass es die zuletzt gesendeten Werte abspeichern soll
* @return Boolean       Hat das Speichern geklappt?
*/
LMSBasicLink.prototype.commit = function() {
    return LMSBasicLink.lms2bool(this.api.LMSCommit(""));
}

// Automatisch bei jedem set() auch commit() ausführen?
LMSBasicLink.prototype._autoCommit = true;
Object.defineProperty(LMSBasicLink.prototype, 'autoCommit', { enumerable: true,
    get() { return this._autoCommit; },
    set(v) {
        v = v ? true : false;
        if (v && (v != this._autoCommit))
            this.commit();
        this._autoCommit = v;
    }
});

/**
* Wert value unter dem Namen key speichern
* @return Boolean       Hat das Speichern geklappt?
*/
LMSBasicLink.prototype.set = function(
    key,        // Name des abzuspeichernden Wertes
    value       // Anzuspeichernde Wert
) {
    if (this.status)
        return false;
    return (LMSBasicLink.lms2bool(this.api.LMSSetValue(key, "" + value)) && (this._autoCommit ? this.commit() : true));
}

/**
* Klasse, die Kommunikation mit einer SCORM-API über Attribute ihrer Instanzen ermöglicht (nach der Initialisierung muss start() aufgerufen werden!)
*/
var LMSAttributeLink = function() {
    LMSAttributeLink.parent.call(this);
}
LMSAttributeLink.prototype = Object.create(LMSBasicLink.prototype);
LMSAttributeLink.prototype.constructor = LMSAttributeLink;
LMSAttributeLink.parent = LMSBasicLink;

/**
* Was soll nach dem Starten sofort gemacht werden?
* @return undefined
*/
LMSAttributeLink.prototype._afterStart = function() {
    switch (this.entry) {
        case 'ab-initio':
            if (typeof this.onnew == 'function')
                this.onnew.call(this);
            break;
        case 'resume':
            if (typeof this.onresume == 'function')
                this.onresume.call(this);
            break;
    }
}

// Funktion, die aufgerufen werden soll, wenn das Lernpaket zum ersten Mal oder neu bearbeitet wird
LMSAttributeLink.prototype.onnew = undefined;

// Funktion, die beim Wiederladen des SCOs ausgeführt werden soll
LMSAttributeLink.prototype.onresume = undefined;

/**
* Ganzzahl v in einen String mit erzungenderweise countDigits Stellen umwandeln
* @return String    Erzeugte Darstellung von v als String
*/
LMSAttributeLink.toDigits = function(
    v,              // Zahl
    countDigits     // Gewünschte Anzahl an Ziffern
) {
    if (v >= Math.pow(10, countDigits))
        v = Math.pow(10, countDigits) - 1;
    if (!Number.isFinite(v))
        v = 0;
    var s = Math.floor(v).toString();
    while (s.length < countDigits)
        s = '0' + s;
    return s;
}

/**
* Eine Anzahl an Sekunden in eine CMITimespan-Angabe umwandeln
* @return String    Berechnete CMITimespan-Angabe
*/
LMSAttributeLink.seconds2timespan = function(
    seconds     // Anzahl an Sekunden, die in eine CMITimespan-Angabe umgewandelt werden soll
) {
    seconds = Math.round(seconds * 100) / 100;
    if (seconds >= 36000000)
        return '9999:99:99.99';
    return LMSAttributeLink.toDigits(seconds / 3600, 4) + ':' + LMSAttributeLink.toDigits((seconds / 60) % 60, 2) + ':' + LMSAttributeLink.toDigits(seconds % 60, 2) + '.' + LMSAttributeLink.toDigits((seconds * 100) % 100, 2);
}

/**
* Wandelt eine CMITimespan-Angabe in die Anzahl an Sekunden um
* @return Number        Anzahl an Sekunden
*/
LMSAttributeLink.timespan2seconds = function(
    strTimespan     // CMITimespan-Angabe der Form "HHHH:MM:SS[.mm]"
) {
    if ((typeof strTimespan != 'string') && !(strTimespan instanceof String))
        return 0;
    var a = strTimespan.split(':').map(function(z) { return (Number.isFinite(+z) ? (+z) : 0); });
    if (a.length > 1)
        a[a.length - 2] *= 60;
    if (a.length > 2)
        a[a.length - 3] *= 3600;
    var v = a.reduce(function(a, b) { return a + b; });
    return (v > 36000060) ? Infinity : v;
}

// Anzahl an Sekunden, die diesmal schon am SCO gearbeitet wurde
Object.defineProperty(LMSAttributeLink.prototype, 'sessionTime', { enumerable: true,
    set(v) {
        this.set('cmi.core.session_time', LMSAttributeLink.seconds2timespan(v));
    }
});

// Anzahl an Sekunden, die zuvor schon insgesamt mit diesem Lernpaket verbracht wurde
Object.defineProperty(LMSAttributeLink.prototype, 'totalTime', { enumerable: true,
    get() {
        return LMSAttributeLink.timespan2seconds(this.get('cmi.core.total_time'));
    }
});

// Maximal erlaubte Bearbeitungszeit in Sekunden
Object.defineProperty(LMSAttributeLink.prototype, 'maxTime', { enumerable: true,
    get() {
        return LMSAttributeLink.timespan2seconds(this.get('cmi.student_data.max_time_allowed'));
    }
});

// Was soll passieren, wenn die maximale Bearbeitungszeit abgelaufen ist?
Object.defineProperty(LMSAttributeLink.prototype, 'timeoutAction', { enumerable: true,
    get() {
        return this.get('cmi.student_data.time_limit_action');
    }
});

// Wird automatisch beendet, wenn die maximale Bearbeitungszeit abgelaufen ist?
Object.defineProperty(LMSAttributeLink.prototype, 'timeoutExit', { enumerable: true,
    get() {
        var s = this.timeoutAction;
        if (s === undefined)
            return undefined;
        return s.length ? (s.indexOf('exit') > -1) : false;
    }
});

// Soll eine Meldung angezeigt werden, wenn die maximale Bearbeitungszeit abgelaufen ist?
Object.defineProperty(LMSAttributeLink.prototype, 'timeoutMessage', { enumerable: true,
    get() {
        var s = this.timeoutAction;
        if (s === undefined)
            return undefined;
        return s.length ? !(s.indexOf('no message') > -1) : false;
    }
});

// ID des Benutzers
Object.defineProperty(LMSAttributeLink.prototype, 'studentID', { enumerable: true,
    get() {
        return this.get('cmi.core.student_id');
    }
});

// Name des Benutzers
Object.defineProperty(LMSAttributeLink.prototype, 'studentName', { enumerable: true,
    get() {
        return this.get('cmi.core.student_name');
    }
});

// Aktueller „Ort“ im SCO
Object.defineProperty(LMSAttributeLink.prototype, 'lessonLocation', { enumerable: true,
    get() { return this.get('cmi.core.lesson_location'); },
    set(v) { this.set('cmi.core.lesson_location', v); }
});

// Aktueller Status des SCO: "passed", "completed", "failed", "incomplete", "browsed", "not attempted"
Object.defineProperty(LMSAttributeLink.prototype, 'lessonStatus', { enumerable: true,
    get() { return this.get('cmi.core.lesson_status'); },
    set(v) { this.set('cmi.core.lesson_status', v); }
});

// Wurde SCO schon einmal aufgerufen?
Object.defineProperty(LMSAttributeLink.prototype, 'entry', { enumerable: true, get() { return this.get('cmi.core.entry'); } });

// Erreichte Punkteanzahl
Object.defineProperty(LMSAttributeLink.prototype, 'score', { enumerable: true,
    get() { var v = this.get('cmi.core.score.raw'); return (v === undefined) ? v : (+v); },
    set(v) { this.set('cmi.core.score.raw', v); }
});

// Minimal erreichbare Punkteanzahl
Object.defineProperty(LMSAttributeLink.prototype, 'scoreMin', { enumerable: true,
    get() { var v = this.get('cmi.core.score.min'); return (v === undefined) ? v : (+v); },
    set(v) { this.set('cmi.core.score.min', v); }
});

// Maximal erreichbare Punkteanzahl
Object.defineProperty(LMSAttributeLink.prototype, 'scoreMax', { enumerable: true,
    get() { var v = this.get('cmi.core.score.max'); return (v === undefined) ? v : (+v); },
    set(v) { this.set('cmi.core.score.max', v); }
});

// Zum Bestehen nötige Punkteanzahl
Object.defineProperty(LMSAttributeLink.prototype, 'scoreMastery', { enumerable: true,
    get() { var v = this.get('cmi.student_data.mastery_score'); return (v === undefined) ? v : (+v); },
});

// Art der Bearbeitung: "browse", "normal" oder "review"
Object.defineProperty(LMSAttributeLink.prototype, 'lessonMode', { enumerable: true,
    get() {
        return this.get('cmi.core.lesson_mode');
    }
});

// Wie das Lernpaket verlassen wird
Object.defineProperty(LMSAttributeLink.prototype, 'exit', { enumerable: true,
    set(v) {
        this.set('cmi.core.exit', v);
    }
});

// Daten (String), welche bei der Weiterbearbeitung des Lernpaketes berücksichtigt werden sollen
Object.defineProperty(LMSAttributeLink.prototype, 'suspendData', { enumerable: true,
    get() { return this.get('cmi.suspend_data'); },
    set(v) { this.set('cmi.suspend_data', v); }
});

// Zusätzlich vom LMS beim Start übergebene Daten (String)
Object.defineProperty(LMSAttributeLink.prototype, 'launchData', { enumerable: true,
    get() { return this.get('cmi.launch_data'); }
});

// Benutzerkommentar
Object.defineProperty(LMSAttributeLink.prototype, 'comments', { enumerable: true,
    get() { return this.get('cmi.comments'); },
    set(v) { this.set('cmi.comments', v); }
});

// Kommentar vom LMS
Object.defineProperty(LMSAttributeLink.prototype, 'commentsFromLms', { enumerable: true,
    get() { return this.get('cmi.comments_from_lms'); }
});

// Gewünschte Lautstärke (Integer)
Object.defineProperty(LMSAttributeLink.prototype, 'preferenceAudio', { enumerable: true,
    get() { var v = this.get('cmi.student_preference.audio'); return (v === undefined) ? v : (+v); },
    set(v) { this.set('cmi.student_preference.audio', v); }
});

// Gewünschte Sprache
Object.defineProperty(LMSAttributeLink.prototype, 'preferenceLanguage', { enumerable: true,
    get() { return this.get('cmi.student_preference.language'); },
    set(v) { this.set('cmi.student_preference.language', v); }
});

// Gewünschte relative Geschwindigkeit (Integer)
Object.defineProperty(LMSAttributeLink.prototype, 'preferenceSpeed', { enumerable: true,
    get() { var v = this.get('cmi.student_preference.speed'); return (v === undefined) ? v : (+v); },
    set(v) { this.set('cmi.student_preference.speed', v); }
});

// Ob und wie Untertitel angezeigt werden sollen (Integer)
Object.defineProperty(LMSAttributeLink.prototype, 'preferenceText', { enumerable: true,
    get() { var v = this.get('cmi.student_preference.text'); return (v === undefined) ? v : (+v); },
    set(v) { this.set('cmi.student_preference.text', v); }
});

/**
* Klasse, die Kommunikation mit einer SCORM-API vereinfacht (nach der Initialisierung muss start() aufgerufen werden!)
*/
var LMSLink = function(
    boolNoAutoStop,     // Optional: Nicht automatisch beim Verlassen der Seite die Kommunikation mit dem LMS beenden
    boolNoTrack         // Optional: Nicht automatisch
) {
    LMSLink.parent.call(this);
    if (!boolNoAutoStop)
        this.addUnloadListener();
    if (!boolNoTrack)
        this.addPauseResumeListener();
}
LMSLink.prototype = Object.create(LMSAttributeLink.prototype);
LMSLink.prototype.constructor = LMSLink;
LMSLink.parent = LMSAttributeLink;

/**
* Dem window-Objekt EventListener für unload und beforeunload hinzufügen, die die Kommunikation mit der API beenden
* @return undefined
*/
LMSLink.prototype.addUnloadListener = function() {
    var self = this;
    var fktUnload = function() { self.stop(); }
    var fktVisibilitychange = function() { self.sessionTime = self.sessionTime; };
    window.addEventListener('unload', fktUnload);
    window.addEventListener('beforeunload', fktUnload);
    window.addEventListener('visibilitychange', fktVisibilitychange);
}

/**
* Dem window-Objekt EventListener für focus und blur hinzufügen, die die Zeitmessung wiederaufnehmen bzw. pausieren
* @return undefined
*/
LMSLink.prototype.addPauseResumeListener = function() {
    var self = this;
    window.addEventListener('blur', function() { self.pause(); });
    window.addEventListener('focus', function() { self.resume(); });
}
/**
* Initialisierung durchführen, sofern dies noch aussteht
* @return Boolean       Wurde (jetzt oder zuvor) eine erfolgreiche Initialisierung durchgeführt
*/
LMSLink.prototype.start = function(
    boolAllwaysResume       // Immer auch this.resume aufrufen, auch dann, wenn das Fenster nicht aktiv ist
) {
    if (this.status >= 0)
        return true;
    if (!this.api)
        return false;
    if (LMSLink.parent.prototype.start.call(this)) {
        var time = LMSAttributeLink.timespan2seconds(this.get('cmi.core.session_time'));
        if (time > 1)
            this._time += time;
        if (boolAllwaysResume || document.hasFocus())
            this.resume();
        return true;
    }
    return false;
}

/**
* Kommunikation mit LMS beenden, sofern dies noch aussteht
* @return Boolean       Wurde (jetzt oder zuvor) die Kommunikation förmlich beendet?
*/
LMSLink.prototype.stop = function() {
    if ((this.status < 0) || !this.api)
        return false;
    if (this.status > 0)
        return true;
    this.pause();
    return LMSLink.parent.prototype.stop.call(this);
}

// Anzahl an Sekunden, die diese SCO bereits genutzt wurde
LMSLink.prototype._time = 0;

// Zeitstempel des Zeitpunktes, an dem die letzte Bearbeitungspause beendet wurde
LMSLink.prototype._lastTimestamp = null;

// Funktion, die beim Starten oder Wiederaufnehmen der Bearbeitung des SCOs ausgeführt werden soll
LMSLink.prototype.onrestart = undefined;

// Funktion, die beim Beenden oder Pausieren der Bearbeitung des SCOs ausgeführt werden soll
LMSLink.prototype.onpause = undefined;

// Funktion, die aufgerufen werden soll, wenn das Lernpaket zum ersten Mal oder neu bearbeitet wird
LMSLink.prototype.onnew = function() {
    this.saveScore(0, 0, 100, 'not attempted');
}

/**
* Bearbeitung des aktuellen SCOs pausieren
* @return undefined
*/
LMSLink.prototype.pause = function() {
    if (this.status)
        return;
    if (typeof this.onpause == 'function')
        this.onpause.call(this);
    if (this._lastTimestamp instanceof Date) {
        this._time += ((new Date()).getTime() - this._lastTimestamp.getTime()) / 1000;
        this._lastTimestamp = null;
    }
    this.sessionTime = this.sessionTime;
}

/**
* Bearbeitung des aktuellen SCOs (wieder-)aufnehmen
* @return undefined
*/
LMSLink.prototype.resume = function() {
    if (this.status)
        return;
    this._lastTimestamp = new Date();
    if (typeof this.onrestart == 'function')
        this.onrestart.call(this);
}

// Ist das Lernmodul gerade aktiv?
Object.defineProperty(LMSLink.prototype, 'active', { enumerable: true,
    get() { return (this._lastTimestamp !== null); },
    set(v) { if (v) this.resume(); else this.pause(); }
});

// Anzahl an Sekunden, die diesmal schon am SCO gearbeitet wurde
Object.defineProperty(LMSLink.prototype, 'sessionTime', { enumerable: true,
    get() {
        var seconds = this._time;
        if (this._lastTimestamp instanceof Date)
            seconds += ((new Date()).getTime() - this._lastTimestamp.getTime()) / 1000;
        return seconds;
    },
    set(v) {
        this.set('cmi.core.session_time', LMSAttributeLink.seconds2timespan(v));
    }
});

// Findet eine Bewertung statt?
Object.defineProperty(LMSLink.prototype, 'isAssessing', { enumerable: true,
    get() {
        v = this.get('cmi.core.credit');
        return (v === undefined) ? v : (v != 'no-credit');
    }
});

// Vorname des Benutzers
Object.defineProperty(LMSLink.prototype, 'studentFirstName', { enumerable: true,
    get() {
        var s = this.studentName;
        if (s === undefined)
            return undefined;
        var index = s.indexOf(',');
        if (index > -1)
            return s.substr(index + 1).trim();
        s = s.trim();
        index = s.indexOf(' ');
        if (index > -1)
            return s.substr(0, index);
        return s;
    }
});

/**
* Bearbeitungszeit speichern
* @return undefined
*/
LMSLink.prototype.reportTime = function() {
    this.sessionTime = this.sessionTime;
}

/**
* Zugleich mehrere Werte ändern, die Zeit speichern und dann erst Commit an das LMS senden
* @return undefined
*/
LMSLink.prototype.save = function(
    data        // Objekt mit zu speichernden Daten
) {
    this.autoCommit = false;
    for (var key in data)
        this[key] = data[key];
    this.reportTime();
    this.autoCommit = true;
}

/**
* Punktestand und Status speichern
* @return undefined
*/
LMSLink.prototype.saveScore = function(
    scoreRaw,       // Erreichter Punktestand
    scoreMin,       // Minimal erreichbarer Punktestand
    scoreMax,       // Maximal erreichbarer Punktestand
    status          // Optional: lessonStatus
) {
    if (this.isNormalIncomplete())
        this.save({
            'lessonStatus': status ? status : 'incomplete',
            'scoreMin': scoreMin,
            'scoreMax': scoreMax,
            'score':    scoreRaw
        });
}

/**
* Ermittelt, ob die Aufgabe bereits abgeschlossen wurde
* @return Boolean       Aufgabe bereits gelöst?
*/
LMSLink.prototype.isIncomplete = function() {
    return (!this.ready || (['completed', 'passed', 'failed'].indexOf(this.lessonStatus) == -1));
}

/**
* Prüft, ob die Aufgabe noch nicht abgeschlossen und das Lernpaket im Normalmodus ist
* @return Boolean       Kann und soll gespeichert werden?
*/
LMSLink.prototype.isNormalIncomplete = function() {
    if (!this.ready)
        return false;
    if (this._canSave === undefined)
        this._canSave = (["browse", "review"].indexOf(this.lessonMode) == -1);
    return (this._canSave && (['completed', 'passed', 'failed'].indexOf(this.lessonStatus) == -1));
}

/**
* HTML-Element in DOM-Baum einfügen
* @return Element   Erzeugter Knoten
*/
function createHtml(
    strTagname,     // Optional: Tagname des zu erzeugenden Elementes
    nodeParent,     // Optional: Künftiger Elternknoten
    objAtts,        // Optional: Attribute des zu erzeugenden Elementes
    strTextContent, // Optional: Textinhalt des Elementes
    nodeBefore      // Optional: Knoten, als deren previousSibling der neue Knoten eingefügt werden soll
) {
    var node = nodeBefore ? nodeBefore.parentNode : nodeParent;
    if (nodeBefore && nodeBefore.parentNode)
        nodeParent = nodeBefore.parentNode;
    if (strTagname)
        node = (nodeParent ? nodeParent.ownerDocument : document).createElementNS('htt' + 'p://w' + 'ww.w3.org/1999/xhtml', strTagname);
    if (objAtts)
        for (var key in objAtts)
            node.setAttribute(key, objAtts[key]);
    if ((strTextContent !== undefined) && ('' + strTextContent).length)
        node.appendChild((nodeParent ? nodeParent.ownerDocument : document).createTextNode(strTextContent));
    if (nodeParent && (node != nodeParent))
        nodeParent.insertBefore(node, nodeBefore);
    return node;
}

/**
* Klasse, die eine Seite modal anzeigt
*/
function ModalPage(
    href,       // URL der anzuzeigenden Seite
    closeLabel  // Optional: Text auf Schließen-Schaltfläche
) {
    Object.defineProperty(this, 'node', {
        enumerable: true,
        writeable: false,
        value: createHtml('div', null, { 'class': 'modal' })
    });
    Object.defineProperty(this, 'iframe', {
        enumerable: true,
        writeable: false,
        value: createHtml('iframe', createHtml('div', createHtml('div', this.node)), { 'src': href })
    });
    var iframe = this.iframe;
    iframe.style.height = '33vh';
    iframe.addEventListener('load', function() {
        try {
            iframe.style.height = Math.min((iframe.contentDocument ? iframe.contentDocument : iframe.contentWindow.document).body.parentNode.scrollHeight + 10, window.innerHeight - 32) + 'px';
        } catch(ex) {}
    });
    Object.defineProperty(this, 'closeBtn', {
        enumerable: true,
        writeable: false,
        value: createHtml('a', createHtml('p', this.iframe.parentNode), { 'href': '#' }, closeLabel ? closeLabel : 'Infokasten schließen')
    });
    var self = this;
    this.closeBtn.addEventListener('click', function(ev) {
        ev.preventDefault();
        ev.stopPropagation();
        self.close();
    });
    this.node.addEventListener('click', function() { self.close() ; });
}

/**
* Modales Fenster anzeigen
* @return undefined
*/
ModalPage.prototype.open = function() {
    document.body.appendChild(this.node);
}

/**
* Modales Fenster ggf. schließen
* @return undefined
*/
ModalPage.prototype.close = function() {
    if (this.node.parentNode)
        this.node.parentNode.removeChild(this.node);
}

/**
* Klasse, die einen Link zu einer modal anzuzeigenden Seite mit weiteren Infos verwaltet
*/
function InfoPageLink(
    node        // Optional: HTMLAnchorElement, welches eine modale Seite öffnen soll
) {
    if (!node) {
        (node = document.getElementsByTagName('footer')[0]) && (node = node.lastChild);
    }
    if (node)
        node.addEventListener('click', function(ev) {
            ev.preventDefault();
            ev.stopPropagation();
            (new ModalPage(this.href)).open();
        });
}

/**
* Eine anklickbare Grafik, die Aktionen auslöst
*/
function ImageLink(
    hint,       // Hinweistext
    src,        // URL der Grafik
    alt,        // Optional: Alternativtext (Standardwert: hint)
    onclick     // Optional: Funktion, die beim Anklicken ausgeführt werden soll
) {
    this.node = ImageLink_createA(this);
    createHtml('img', this.node, { 'src': src, 'alt': alt ? alt : hint });
    this.hint = hint;
    if (onclick)
        this.onclick = onclick;
}

/**
* Neues A-Element erstellen
* @return HTMLAnchorElement
*/
function ImageLink_createA(
    self        // Zuständige ImageLink-Instanz
) {
    var node = createHtml('a');
    node.setAttribute('href', '#');
    node.setAttribute('role', 'button');
    node.className = 'ImageLink';
    node.addEventListener('click', function(ev) {
        ev.preventDefault();
        ev.stopPropagation();
        self.click();
    });
    return node;
}

// Aktuelles HTML-Element
ImageLink.prototype.node = null;

// Deaktiviert?
Object.defineProperty(ImageLink.prototype, 'disabled', {
    enumerable: true,
    get() {
        return (this.node.nodeName.toUpperCase() != 'A');
    },
    set(v) {
        var node = null;
        if (this.disabled) {
            if (v)
                return;
            node = ImageLink_createA(this);
        } else {
            if (!v)
                return;
            node = createHtml('span', null, { 'class': 'ImageLink', 'role': 'button', 'aria-disabled': 'aria-disabled' });
        }
        if (this.node.hasAttribute('title'))
            node.setAttribute('title', this.node.getAttribute('title'));
        node.appendChild(this.node.firstChild);
        if (this.node.parentNode)
            this.node.parentNode.replaceChild(node, this.node);
        this.node = node;
    }
});

// URL der Grafik
Object.defineProperty(ImageLink.prototype, 'src', {
    enumerable: true,
    get() {
        return this.node.firstChild.src;
    },
    set(s) {
        this.node.firstChild.src = s;
    }
});

// Alternativtext der Grafik
Object.defineProperty(ImageLink.prototype, 'alt', {
    enumerable: true,
    get() {
        return this.node.firstChild.alt;
    },
    set(s) {
        this.node.firstChild.alt = s;
    }
});

// Hinweistext
Object.defineProperty(ImageLink.prototype, 'hint', {
    enumerable: true,
    get() {
        return (this.node.getAttribute('title') || '');
    },
    set(s) {
        if (s)
            this.node.setAttribute('title', s);
        else
            this.node.removeAttribute('title');
    }
});

// Was beim Anklicken passieren soll
ImageLink.prototype.onclick = null;

/**
* Auf Anklicken reagieren
* @return undefined
*/
ImageLink.prototype.click = function() {
    if (typeof this.onclick == 'function')
        this.onclick.call(this);
}

/**
* Die ImageLink-Instanz old durch this ersetzen
* @return undefined
*/
ImageLink.prototype.replace = function(
    old     // ImageLink-Instanz, die ersetzt werden soll
) {
    if ((old instanceof ImageLink) && (old != this) && (old.node.parentNode))
        old.node.parentNode.replaceChild(this.node, old.node);
}

/**
* Diesen Link als Kind von parentNode im DOM-Baum einfügen
* @return undefined
*/
ImageLink.prototype.appendAsChild = function(
    parentNode      // Neuer Elternknoten
) {
    parentNode.appendChild(this.node);
}

/**
* Eine Meldung, die dem Benutzer angezeigt wird
*/
function Message(
    text,       // Meldungstext
    type        // Optional: Meldungsart (Standardwert: Message.TYPE_FAILED)
) {
    Object.defineProperty(this, 'node', {
        enumerable: true,
        value: createHtml('p')
    });
    createHtml('span', createHtml('span', this.node));
    this.type = type;
    if (text)
        this.text = text;
}

// Mögliche Meldungsarten
Object.defineProperty(Message, 'TYPE_SUCCESS', { enumerable: true, value: 'msg_success' });
Object.defineProperty(Message, 'TYPE_FAILED', { enumerable: true, value: 'msg_failed' });
Object.defineProperty(Message, 'TYPE_WARN', { enumerable: true, value: 'msg_warn' });
Object.defineProperty(Message, 'TYPE_INFO', { enumerable: true, value: 'msg_info' });
Object.defineProperty(Message, 'TYPES', { enumerable: true, value: [ Message.TYPE_FAILED, Message.TYPE_SUCCESS, Message.TYPE_INFO, Message.TYPE_WARN ] });

// Meldungsart
Object.defineProperty(Message.prototype, 'type', {
    enumerable: true,
    get() {
        return this.node.className;
    },
    set(s) {
        if (Message.TYPES.indexOf(s) == -1)
            s = s ? Message.TYPE_SUCCESS : Message.TYPE_FAILED;
        this.node.className = s;
    }
});

// Meldungstext
Object.defineProperty(Message.prototype, 'text', {
    enumerable: true,
    get() {
        return this.node.firstChild.firstChild.textContent;
    },
    set(v) {
        this.node.firstChild.firstChild.textContent = v;
    }
});

// Genutzte MessagePlace-Instanz
Message.prototype._place = null;
Object.defineProperty(Message.prototype, 'place', {
    enumerable: true,
    get() {
        return this._place;
    },
    set(place) {
        this.addTo(place);
    }
});

/**
* Diese Meldung auf MessagePlace-Instanz place anzeigen
* @return undefined
*/
Message.prototype.addTo = function(
    place       // MessagePlace-Instanz, auf der die Meldung exklusiv angezeigt werden soll
) {
    this.remove();
    if (place instanceof MessagePlace) {
        place.register(this);
        this._place = place;
    } else
        this._place = null;
}

/**
* Diese Meldung von der Anzeigefläche entfernen
* @return undefined
*/
Message.prototype.remove = function() {
    if (this._place)
        this._place.unregister(this);
    this._place = null;
}

/**
* Platz für Meldungen
*/
function MessagePlace(
    multiple        // Optional: Sollen mehrere Meldungen angezeigt werden können?
) {
    Object.defineProperty(this, 'node', {
        enumerable: true,
        value: createHtml('div', null, { 'class': 'messagePlace' })
    });
    this.messages = [];
    if (multiple)
        this.multiple = true;
}

// Angezeigte Meldungen
MessagePlace.prototype.messages = [];

// Sollen mehrere Meldungen angezeigt werden können?
MessagePlace.prototype.multiple = false;

/**
* Platz für Meldungen im DOM-Baum vor nodeAfter einfügen
* @return undefined
*/
MessagePlace.prototype.insertThisBefore = function(
    nodeAfter       // Knoten, vor dem eingefügt werden soll
) {
    nodeAfter.parentNode.insertBefore(this.node, nodeAfter);
}

/**
* Platz für Meldungen im DOM-Baum nach nodeBefore einfügen
* @return undefined
*/
MessagePlace.prototype.insertThisAfter = function(
    nodeBefore      // Knoten, nach dem eingefügt werden soll
) {
    nodeBefore.parentNode.insertBefore(this.node, nodeBefore.nextSibling);
}

/**
* Platz für Meldungen im DOM-Baum als letztes Kind von parentNode einfügen
* @return undefined
*/
MessagePlace.prototype.appendAsChild = function(
    parentNode      // Künftiger Elternknoten
) {
    parentNode.appendChild(this.node);
}

/**
* Message-Instanz msg als eigene Nachricht registrieren und in DOM-Baum einfügen
* @return undefined
*/
MessagePlace.prototype.register = function(
    msg         // Zu registerende Message-Instanz
) {
    if (!(msg instanceof Message))
        return;
    this.node.appendChild(msg.node);
    this.messages.push(msg);
}

/**
* Message-Instanz msg von ihrer MessagePlace-Instanz und aus dem DOM-Baum entfernen
* @return undefined
*/
MessagePlace.prototype.unregister = function(
    msg         // Zu entferndende Message-Instanz
) {
    if (!(msg instanceof Message))
        return;
    this.messages = this.messages.filter(function(o) { return (o != msg); });
    if (msg.node.parentNode)
        msg.node.parentNode.removeChild(msg.node);
}

/**
* Alle aktuell angezeigten Meldungen entfernen
* @return undefined
*/
MessagePlace.prototype.clear = function() {
    this.messages.map(function(msg) { msg.remove(); });
    this.messages = [];
}

/**
* Eine Message-Instanz msg anzeigen
* @return undefined
*/
MessagePlace.prototype.append = function(
    msg         // Anzuzeigende Message-Instanz
) {
    if (!(msg instanceof Message))
        return;
    if (!this.multiple)
        this.clear();
    msg.addTo(this);
}

/**
* Eine Meldung anlegen und anzeigen
* @return Message   Erstellte Meldung
*/
MessagePlace.prototype.add = function(
    text,       // Meldungstext
    type        // Optional: Meldungsart (Standardwert: Message.TYPE_FAILED)
) {
    var msg = new Message(text, type);
    this.append(msg);
    return msg;
}

/**
* Klasse, die einen zusätzlichen Abschnitt im Footer verwaltet
*/
function FooterSection() {
    FooterSection.object = this;
    var footer = document.getElementsByTagName('footer');
    if (footer.length)
        footer = footer[footer.length - 1];
    else
        footer = createHtml('footer', document.body);
    this.node = createHtml('span', footer, null, undefined, footer.firstChild);
}

/**
* Bisherige FooterSection-Instanz aufrufen oder eine FooterSection-Instanz erzeugen, wenn es noch keine gibt
* @return FooterSection     FooterSection-Instanz
*/
FooterSection.getObject = function() {
    if (!FooterSection.object)
        FooterSection.object = new FooterSection();
    return FooterSection.object;
}

// DOM-Repräsentation
FooterSection.prototype.node = null;

/**
* Die ImageLink-Instanz oder den DOM-Knoten obj aufnehmen
* @return undefined
*/
FooterSection.prototype.appendChild = function(
    obj         // Aufzunehmende ImageLink-Instanz oder DOM-Knoten
) {
    if (obj instanceof ImageLink)
        obj = obj.node;
    this.node.appendChild(obj);
    this.node.appendChild(document.createTextNode(' '));
}

/**
* Die ImageLink-Instanz oder den DOM-Knoten obj entfernen
* @return undefined
*/
FooterSection.prototype.removeChild = function(
    obj         // Aufzunehmende ImageLink-Instanz oder DOM-Knoten
) {
    if (obj instanceof ImageLink)
        obj = obj.node;
    if (obj.parentNode != this.node)
        return;
    var next = obj.nextSibling;
    this.node.removeChild(obj);
    if (next)
        this.node.removeChild(next);
}


ScoTools.LMSBasicLink = LMSBasicLink;
ScoTools.LMSAttributeLink = LMSAttributeLink;
ScoTools.LMSLink = LMSLink;
ScoTools.createHtml = createHtml;
ScoTools.ModalPage = ModalPage;
ScoTools.InfoPageLink = InfoPageLink;
ScoTools.ImageLink = ImageLink;
ScoTools.Message = Message;
Object.defineProperty(ScoTools, 'MSGTYPE_SUCCESS', { enumerable: true, value: Message.TYPE_SUCCESS });
Object.defineProperty(ScoTools, 'MSGTYPE_FAILED', { enumerable: true, value: Message.TYPE_FAILED });
Object.defineProperty(ScoTools, 'MSGTYPE_INFO', { enumerable: true, value: Message.TYPE_INFO });
Object.defineProperty(ScoTools, 'MSGTYPE_WARN', { enumerable: true, value: Message.TYPE_WARN });
Object.defineProperty(ScoTools, 'MSGTYPES', { enumerable: true, value: Message.TYPES });
ScoTools.MessagePlace = MessagePlace;
ScoTools.FooterSection = FooterSection;
ScoTools.getFooterSection = FooterSection.getObject;
return ScoTools;
})();
window.createHtml = ScoTools.createHtml;

